#include "int_list.h"
#include <stdlib.h>
#include <stdio.h>

/* Report an error and exit the program with a failure */
void report_error(char *msg){
  fprintf(stderr,"int_list: %s\n",msg);
  exit(-1);
}

// Create a list of size 0 and return it
int_list* int_list_create(){
  int_list *new = malloc(sizeof(int_list));
  new->head = NULL;
  return new;
}

// Destroy int_list
void int_list_destroy(int_list* list){
  int_node *ptr = list->head;
  while(ptr != NULL){
    int_node *freeit = ptr;
    ptr = ptr->next;
    free(freeit);
  }
  free(list);
  return;
}

/* Returns how many nodes are in a list */
int int_list_size(int_list *l){
  int n = 0;
  int_node *ptr = l->head;
  while(ptr != NULL){
    n++;
    ptr = ptr->next;
  }
  return n;
}

int int_list_get(int_list *l, int index){
  int i = 0;
  int_node *ptr = l->head;
  while(ptr != NULL && i!=index){
    i++;
    ptr = ptr->next;
  }
  if(i!=index){
    report_error("Index out of bounds in set()\n");
  }
  int value = ptr->data;
  return value;
}

// Set element at specified index
void int_list_set(int_list *l, int index,
                  int new_val)
{
  int i = 0;
  int_node *ptr = l->head;
  while(ptr != NULL && i!=index){
    i++;
    ptr = ptr->next;
  }
  if(i!=index){
    report_error("Index out of bounds in set()\n");
  }
  ptr->data = new_val;
}


// Prepend a new element to the beginning of the list
void int_list_prepend(int_list *list, int new_elem){
  int_node *new = malloc(sizeof(int_node));
  new->data = new_elem;
  new->next = list->head;
  list->head = new;
  return;
}

void int_list_print(int_list *list, FILE* fout){
  int_node *ptr = list->head;
  int i=0;
  if(ptr == NULL){
    fprintf(fout,"Empty");
    return;
  }
  while(ptr != NULL){
    fprintf(fout,"%d:%d ",i,ptr->data);
    ptr = ptr->next;
    i++;
  }
  return;
}


/* Remove the node at given index */
int int_list_remove(int_list *l, int index){
  if(l->head == NULL){ /* Nothing to remove */
    return 0;
  }
  /* Removing head is a special case */
  if(index == 0){
    int_node *remove = l->head;
    l->head = l->head->next;
    free(remove);
    return 1;
  }
  /* Remove from the middle */
  int i;
  int_node *ptr = l->head;
  for(i=0; i<index-1; i++){
    if(ptr == NULL){
      return 0;                 /* Unsuccessful: list too short */
    }
    ptr=ptr->next;
  }
  /* ptr is now node preceding removal point */
  int_node *remove = ptr->next;
  if(remove == NULL){           /* Remove might still be out of bounds */
    return 0;
  }
  ptr->next = remove->next;
  free(remove);
  return 1;
}

// Get element at specified index
int int_list_get(int_list *l, int index){
  int i = 0;
  int_node *ptr = l->head;
  while(ptr != NULL && i!=index){
    i++;
    ptr = ptr->next;
  }
  if(i!=index){
    report_error("Index out of bounds in set()\n");
  }
  return ptr->data;
}
